ProvableCircuit

Struct ProvableCircuit 

pub struct ProvableCircuit<F: Halo2FFTFriendlyField> {
    circuit_description: GKRCircuitDescription<F>,
    inputs: HashMap<LayerId, MultilinearExtension<F>>,
    committed_inputs: HashMap<LayerId, LigeroInputLayerDescriptionWithOptionalProverPrecommit<F>>,
    layer_label_to_layer_id: HashMap<String, LayerId>,
}
A circuit, along with all of its input data, ready to be proven using the vanila GKR proving system which uses Ligero as a PCS for committed input layers, and provides no zero-knowledge guarantees.

Fields

circuit_description: GKRCircuitDescription<F>inputs: HashMap<LayerId, MultilinearExtension<F>>committed_inputs: HashMap<LayerId, LigeroInputLayerDescriptionWithOptionalProverPrecommit<F>>layer_label_to_layer_id: HashMap<String, LayerId>

Implementations

Source

impl<F: Halo2FFTFriendlyField> ProvableCircuit<F>

pub fn new( circuit_description: GKRCircuitDescription<F>, inputs: HashMap<LayerId, MultilinearExtension<F>>, committed_inputs: HashMap<LayerId, LigeroInputLayerDescriptionWithOptionalProverPrecommit<F>>, layer_label_to_layer_id: HashMap<String, LayerId>, ) -> Self

Constructor

pub fn _gen_verifiable_circuit(&self) -> VerifiableCircuit<F>

WARNING

To be used only for testing and debugging.

Constructs a form of this circuit that can be verified when a proof is provided. This is done by erasing all input data associated with committed input layers, along with any commitments (the latter can be found in the proof).

pub fn get_gkr_circuit_description_ref(&self) -> &GKRCircuitDescription<F>

Returns a reference to the GKRCircuitDescription of this circuit.

TODO: This is only used by the back end. Do not expose it to the circuit developer.

pub fn get_public_input_layer_ids(&self) -> Vec<LayerId>

Returns a vector of the LayerIds of all input layers which are public.

TODO: Consider returning an iterator instead.

pub fn get_committed_input_layer_ids(&self) -> Vec<LayerId>

Returns a vector of the LayerIds of all input layers which are committed to using a PCS.

TODO: Consider returning an iterator instead.

pub fn get_input_mle( &self, layer_id: LayerId, ) -> Result<MultilinearExtension<F>>

Returns the data associated with the input layer with ID layer_id, or an error if there is no input layer with this ID.

pub fn get_committed_input_layer( &self, layer_id: LayerId, ) -> Result<LigeroInputLayerDescriptionWithOptionalProverPrecommit<F>>

Returns the description of the committed input layer with ID layer_id, or an error if no such committed input layer exists.

pub fn get_inputs_ref(&self) -> &HashMap<LayerId, MultilinearExtension<F>>

Get a reference to the mapping that maps a LayerId of layer (either public or committed) to the data that are associated with it.

TODO: This is too transparent. Replace this with methods that answer the queries of the prover directly, and do not expose it to the circuit developer.

pub fn prove<Tr: TranscriptSponge<F>>( &self, circuit_description_hash_type: CircuitHashType, transcript_writer: &mut TranscriptWriter<F, Tr>, ) -> Result<ProofConfig>

Write the GKR proof to transcript. Appends inputs and Ligero input layer commitments to the transcript: public inputs first, then Ligero input layers, ordering by layer id in both cases. Arguments:

  • circuit_description_hash_type: The hash function we wish to use to hash the circuit description.
  • transcript_writer: The struct used by the prover to write messages to the transcript, which will be used by the verifier to verify the circuit.